// Include windows.h
#define WIN32_LEAN_AND_MEAN
#include <windows.h>

// .NET Framework goo
#using <mscorlib.dll>
using namespace System;
using namespace System::Collections;

// DLL Project specific goo
#define NETWRAPPER_LINKAGE __declspec(dllexport)
#include "NetWrapper.h"

// Just one of many ways to keep track of managed objects
__gc class ManagedObjects{
public:
   static Hashtable* table;
};

// Good ol' DllMain()
BOOL APIENTRY 
   DllMain(HANDLE hModule, DWORD dwReason, LPVOID lpReserved)
{
   Hashtable* pTable = NULL; // temp reference

   if(dwReason == DLL_PROCESS_ATTACH){
      pTable = new Hashtable(); // Create a Hashtable on DLL load
      
      // Get a thread synchronized wrapper around the object
      ManagedObjects::table = Hashtable::Synchronized(pTable);   
   }
   return TRUE;
}

// Helper function to translate an HRAND into an object reference
inline Random* GetRandomObj(HRAND handle){
   Random* obj = dynamic_cast<Random*>
      (ManagedObjects::table->get_Item(__box(handle)));
   return obj;
}

// Exported DLL Functions
extern "C"{

// Create a Random object
__declspec(dllexport) HRAND RandomCreate(){
   Random* rand = new Random(); // Managed object

   // Get the hashcode that we will use as a "handle"
   int handle = rand->GetHashCode();
   
   // Add the object to the hashtable
   ManagedObjects::table->Add(__box(handle), rand);
   return (HRAND)handle; // return "handle"
}

// Wrapper around the Random::Next(int maxValue) method
__declspec(dllexport) int RandomNext(HRAND handle, int maxValue){
   Random* rand = GetRandomObj(handle); // Get the object
   return rand->Next(maxValue); // call managed method
}

__declspec(dllexport) 
   void RandomNextBytes(HRAND handle, PBYTE buffer, DWORD size){
   Random* rand = GetRandomObj(handle); // Get object

   // Create managed array of Byte's
   Byte tempBytes __gc[] = new Byte __gc[size];
   rand->NextBytes(tempBytes); // Call managed method to fill array
   
   // Pin managed array and copy memory
   BYTE __pin *pinBuffer = &(tempBytes[0]); 
   CopyMemory(buffer, pinBuffer, size);
   pinBuffer = NULL; // null out pinned reference
}

__declspec(dllexport) void RandomClose(HRAND handle){
   // Unreference the managed object so it can be garbage collected
   ManagedObjects::table->Remove(__box(handle));
}

}
